var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { ProviderRpcError, webSocket, publicActions, http, formatTransaction, } from 'viem';
import { EventEmitter } from 'eventemitter3';
import { extractRpcUrls } from '@wagmi/core';
import { getViemChain, createCapsuleViemClient, createCapsuleAccount } from '@usecapsule/viem-v2-integration';
import { decimalToHex, hexToDecimal } from '@usecapsule/react-sdk';
import { renderModal } from './connectorModal.js';
const STORAGE_CHAIN_ID_KEY = '@CAPSULE/chainId';
const TEN_MINUTES_MS = 600000;
const serverSessionStorageStub = {
    setItem: () => { },
    getItem: () => null,
};
export class CapsuleEIP1193Provider extends EventEmitter {
    constructor(opts) {
        super();
        this.getRpcUrlsFromViemChain = (chain) => {
            return extractRpcUrls({ chain, transports: this.transports });
        };
        this.wagmiChainToAddEthereumChainParameters = (chain) => {
            const hexChainId = decimalToHex(`${chain.id}`);
            return [
                hexChainId,
                {
                    chainId: hexChainId,
                    chainName: chain.name,
                    nativeCurrency: chain.nativeCurrency,
                    rpcUrls: this.getRpcUrlsFromViemChain(chain),
                },
            ];
        };
        this.wagmiChainsToAddEthereumChainParameters = (chains) => {
            return Object.fromEntries(chains.map(this.wagmiChainToAddEthereumChainParameters));
        };
        this.accountFromAddress = (address) => {
            return createCapsuleAccount(this.capsule, address);
        };
        this.setCurrentChain = (chainId) => {
            const chain = this.chains[chainId];
            this.setChainId(chainId);
            const viemChain = this.viemChains[chainId] || getViemChain(hexToDecimal(chainId));
            const rpcUrls = this.getRpcUrlsFromViemChain(viemChain);
            let transport;
            if (this.transports[viemChain.id]) {
                transport = this.transports[viemChain.id];
            }
            else if (rpcUrls[0].startsWith('ws')) {
                transport = webSocket(chain.rpcUrls[0]);
            }
            else {
                transport = http(rpcUrls[0]);
                this.chainTransportSubscribe = undefined;
            }
            const chainTransport = transport({
                chain: viemChain,
            });
            if (chainTransport.config.type === 'ws') {
                this.chainTransportSubscribe = chainTransport.value.subscribe;
            }
            this.walletClient = createCapsuleViemClient(this.capsule, {
                chain: viemChain,
                transport,
                // @ts-ignore
            }, { noAccount: true }).extend(publicActions);
            this.emit('chainChanged', this.currentHexChainId);
        };
        this.closeModal = () => {
            this.isModalClosed = true;
        };
        this.request = (args) => __awaiter(this, void 0, void 0, function* () {
            const { method, params } = args;
            switch (method) {
                case 'eth_accounts': {
                    return this.accounts;
                }
                case 'eth_chainId': {
                    return this.currentHexChainId;
                }
                case 'eth_requestAccounts': {
                    if (yield this.capsule.isFullyLoggedIn()) {
                        return this.accounts;
                    }
                    this.isModalClosed = false;
                    const onClose = () => {
                        this.isModalClosed = true;
                    };
                    if (!this.disableModal) {
                        renderModal(this.capsule, this.modalProps, onClose);
                    }
                    // check if capsule is fully logged in every 2 seconds for 10 minutes at most
                    const now = Date.now();
                    while (Date.now() - now < TEN_MINUTES_MS) {
                        yield new Promise(resolve => setTimeout(resolve, 2000));
                        if (yield this.capsule.isFullyLoggedIn()) {
                            const accounts = this.accounts;
                            this.emit('accountsChanged', accounts);
                            return accounts;
                        }
                        if (this.isModalClosed) {
                            throw new ProviderRpcError(new Error('user closed modal'), {
                                code: 4001,
                                shortMessage: 'user closed modal',
                            });
                        }
                    }
                    throw new ProviderRpcError(new Error('timed out waiting for user to log in'), {
                        code: 4001,
                        shortMessage: 'timed out waiting for user to log in',
                    });
                }
                case 'eth_sendTransaction': {
                    const fromAddress = params[0].from;
                    return this.walletClient.sendTransaction(Object.assign(Object.assign({}, formatTransaction(params[0])), { chain: undefined, account: this.accountFromAddress(fromAddress) }));
                }
                case 'eth_sign':
                case 'personal_sign': {
                    return this.walletClient.signMessage({
                        message: { raw: params[0] },
                        account: this.accountFromAddress(params[1]),
                    });
                }
                case 'eth_signTransaction': {
                    const fromAddress = params[0].from;
                    return this.accountFromAddress(fromAddress).signTransaction(formatTransaction(params[0]));
                }
                case 'eth_signTypedData_v4': {
                    const fromAddress = params[0];
                    let typedMessage = params[1];
                    if (typeof typedMessage === 'string') {
                        typedMessage = JSON.parse(typedMessage);
                    }
                    return this.walletClient.signTypedData(Object.assign(Object.assign({}, typedMessage), { account: this.accountFromAddress(fromAddress) }));
                }
                case 'eth_subscribe': {
                    if (!this.chainTransportSubscribe) {
                        throw new ProviderRpcError(new Error('chain does not support subscriptions'), {
                            code: 4200,
                            shortMessage: 'chain does not support subscriptions',
                        });
                    }
                    const res = yield this.chainTransportSubscribe({
                        params: params,
                        onData: data => {
                            this.emit('message', {
                                type: 'eth_subscription',
                                data,
                            });
                        },
                    });
                    return res.subscriptionId;
                }
                case 'wallet_addEthereumChain': {
                    if (!this.chains[params[0].chainId]) {
                        this.chains[params[0].chainId] = params[0];
                    }
                    return null;
                }
                case 'wallet_getPermissions': {
                    // capsule doesn't support this type of functionality for now
                    return [];
                }
                case 'wallet_requestPermissions': {
                    // capsule doesn't support this type of functionality for now
                    return [];
                }
                case 'wallet_switchEthereumChain': {
                    if (!this.chains[params[0].chainId]) {
                        const chain = getViemChain(hexToDecimal(params[0].chainId));
                        const [hexChainId, addEthereumChainParameter] = this.wagmiChainToAddEthereumChainParameters(chain);
                        this.chains[hexChainId] = addEthereumChainParameter;
                        this.setCurrentChain(params[0].chainId);
                    }
                    if (this.currentHexChainId !== params[0].chainId) {
                        this.setCurrentChain(params[0].chainId);
                    }
                    return null;
                }
                case 'wallet_watchAsset': {
                    // capsule doesn't support this type of functionality for now
                    return false;
                }
                default: {
                    return this.walletClient.request({
                        method,
                        params,
                    });
                }
            }
        });
        this.storage = opts.storageOverride || typeof window === 'undefined' ? serverSessionStorageStub : sessionStorage;
        this.isModalClosed = true;
        this.capsule = opts.capsule;
        this.modalProps = Object.assign({}, opts);
        this.disableModal = !!opts.disableModal;
        this.viemChains = opts.chains.reduce((acc, curChain) => {
            acc[decimalToHex(`${curChain.id}`)] = curChain;
            return acc;
        }, {});
        this.chains = this.wagmiChainsToAddEthereumChainParameters(opts.chains);
        this.transports = opts.transports;
        const defaultChainId = this.getStorageChainId() || opts.chainId;
        const currentChainId = this.chains[decimalToHex(defaultChainId)] ? defaultChainId : `${opts.chains[0].id}`;
        this.setCurrentChain(decimalToHex(currentChainId));
        this.emit('connect', { chainId: this.currentHexChainId });
    }
    get accounts() {
        return this.capsule.getWalletsByType('EVM').map(w => w.address);
    }
    getStorageChainId() {
        return this.storage.getItem(STORAGE_CHAIN_ID_KEY);
    }
    setChainId(hexChainId) {
        this.currentHexChainId = hexChainId;
        this.storage.setItem(STORAGE_CHAIN_ID_KEY, hexToDecimal(hexChainId));
    }
}
